from django.contrib.db import models

from . import defaults as defs


class CountryField(models.CharField):
    """ Custom Country Filed """
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', 2)
        kwargs.setdefault('choices', defs.COUNTRY_CODES)
        super().__init__(*args, **kwargs)

    def get_internal_type(self):
        return "CharField"



import time

from django.test import TestCase
from django.conf import settings
from django.utils import translation
from django.core.management import call_command

from countryware.country import get_all_countries
from countryware.country import get_all_countries_sorted
from countryware.country import get_all_countries_prioritized
from countryware.country import get_display
from countryware import defaults as defs


class TestCountryCase(TestCase):
    """
    Country Test
    """
    def setUp(self):
        # call_command('compilemessages')
        self.andorra = defs.ALL_COUNTRY_CODES[0]

    def test_xlate_display(self):
        name = get_display('AD')
        self.assertEquals(name, 'Andorra')

    def test_xlate_fa(self):
        translation.activate('fa')
        name = get_display('AD')
        self.assertEquals(name, 'آندورا')

    def test_xlate_fa(self):
        translation.activate('he')
        name = get_display('AD')
        self.assertEquals(name, 'אנדורה')

    def test_xlate_priority(self):
        translation.activate('zh_Hans')
        name = get_display('AD')
        self.assertEquals(name, '安道尔')

    def test_xlate_en_unsorted(self):
        translation.activate('en')
        countries = get_all_countries()
        self.assertEquals(countries[0][1], 'Andorra')

    def test_xlate_en_sorted(self):
        translation.activate('en')
        countries = get_all_countries_sorted()
        self.assertEquals(countries[0][1], 'Afghanistan')

    def test_xlate_en_prioritized(self):
        translation.activate('en')
        countries = get_all_countries_prioritized()
        self.assertEquals(countries[0][1], 'Canada')

    def test_xlate_fa_prioritized(self):
        translation.activate('fa')
        countries = get_all_countries_prioritized()
        self.assertEquals(countries[0][1], 'کانادا')


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}
SECRET_KEY = "un33k"
INSTALLED_APPS = ['countryware']
MIDDLEWARE_CLASSES = []
PRIORITY_COUNTRY_CODES = ['CA']


import locale
import functools

from django.utils import translation
from django.utils.translation import ugettext as _

from . import defaults as defs
from .utils import memorize

_cache_countries = {}
_cache_countries_sorted = {}
_cache_countries_sorted_prioritized = {}


def get_display(code):
    # Note: admin:skip
    display = _('ISO_3166-1.' + code)
    return display


def get_countries(codes):
    """ Returns a list of (code, translation) tuples for codes  """
    countries = [(code, get_display(code)) for code in codes]
    return countries


@memorize(_cache_countries)
def get_all_countries(codes=defs.ALL_COUNTRY_CODES):
    """ Returns a list of (code, translation) tuples for all country codes  """
    countries = get_countries(codes)
    return countries


@memorize(_cache_countries_sorted)
def get_all_countries_sorted(codes=defs.ALL_COUNTRY_CODES):
    """ Returns a list of (code, translation) tuples for all country codes  """
    countries = sorted(
        get_countries(codes),
        key=functools.cmp_to_key(lambda a, b: locale.strcoll(a[1], b[1]))
    )
    return countries


@memorize(_cache_countries_sorted_prioritized)
def get_all_countries_prioritized(
        priority_codes=defs.PRIORITY_COUNTRY_CODES,
        codes=defs.ALL_COUNTRY_CODES
    ):
    """ Returns a sorted list of (code, translation) tuples for codes  """
    prioritized = []
    if (priority_codes and len(priority_codes) > 0):
        prioritized = get_countries(priority_codes)
    countries = get_all_countries(codes)
    return prioritized + countries


import inspect

from django.core.cache import cache
from django.utils import translation
from django.utils.translation import ugettext as _

_cache_backend_enabled = None


def check_cache():    
    """ Check if a cache backend is available """
    global _cache_backend_enabled
    if _cache_backend_enabled is None:
        value = 100
        key = 'testing-if-cache-is-enabled'
        try:
            cache.set(key, value)
            if cache.get(key) == value:
                _cache_backend_enabled = True
        except Exception:
            _cache_backend_enabled = False


def get_from_cache(backend, key):
    """ get value for key from cache """
    if _cache_backend_enabled is True:
        return cache.get(key)
    return backend.get(key)


def set_to_cache(backend, key, value):
    """ set value for key to cache """
    if _cache_backend_enabled is True:
        cache.set(key, value)
    else:
        backend[key] = value


def get_cache_key(backend):
    """ get cache key for operation """
    lang = translation.get_language()
    key = '{}-{}'.format(id(backend), lang)
    return key


def memorize(cache_storage):
    """ decorator to cache operation """
    def decorator(function):
        def wrapper(*args, **kwargs):
            check_cache()
            key = get_cache_key(cache_storage)
            result = get_from_cache(cache_storage, key)
            if result is None:
                result = function(*args, **kwargs)
                set_to_cache(cache_storage, key, result)
            return result
        return wrapper
    return decorator
